"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const boa = require('@pipcook/boa');
const { enumerate, list, len, dict } = boa.builtins();
const torch = boa.import('torch');
const { DataLoader, Dataset } = boa.import('torch.utils.data');
const Image = boa.import('PIL.Image');
const np = boa.import('numpy');
const transforms = boa.import('torchvision.transforms');
/**
 * @ignore
 * create custom dataset
 */
const getDataSet = (dataLoader) => __awaiter(void 0, void 0, void 0, function* () {
    const data = [];
    const count = yield dataLoader.len();
    for (let i = 0; i < count; i++) {
        const currentData = yield dataLoader.getItem(i);
        if (fs.existsSync(currentData.data)) {
            data.push({
                fileName: currentData.data,
                className: currentData.label.categoryId
            });
        }
    }
    const transform = transforms.Compose([transforms.ToTensor()]);
    class ImageClassificationData extends Dataset {
        constructor(transform) {
            super();
            this.data = list(data);
            this.transform = transform;
        }
        __len__() {
            return len(this.data);
        }
        __getitem__(index) {
            if (torch.is_tensor(index)) {
                index = index.tolist();
            }
            const imageData = this.data[index];
            let image = np.array(Image.open(imageData['fileName']));
            if (this.transform) {
                image = this.transform(image);
            }
            const label = imageData['className'];
            return dict({
                image,
                label
            });
        }
    }
    return new ImageClassificationData(transform);
});
const modelTrain = (data, model, args) => __awaiter(void 0, void 0, void 0, function* () {
    const { epochs = 10, batchSize = 16, saveModel, printEvery = 100 } = args;
    const { trainLoader, validationLoader } = data;
    let valDataLoader;
    if (validationLoader) {
        const val_dataset = yield getDataSet(validationLoader);
        valDataLoader = DataLoader(val_dataset, boa.kwargs({
            batch_size: batchSize
        }));
    }
    if (trainLoader) {
        const train_dataset = yield getDataSet(trainLoader);
        const dataloader = DataLoader(train_dataset, boa.kwargs({
            batch_size: batchSize
        }));
        for (let i = 0; i < epochs; i++) {
            model.model.train();
            let running_loss = 0;
            enumerate(dataloader, 0).forEach((data, steps) => {
                let inputs = data['image'];
                let labels = data['label'];
                model.optimizer.zero_grad();
                const outputs = model.model(inputs);
                const loss = model.criterion(outputs, labels);
                loss.backward();
                model.optimizer.step();
                running_loss += loss.item();
                if (steps % printEvery === 0) {
                    console.log(`[epoch ${i}, step ${steps}] training loss: ${running_loss / printEvery}`);
                    running_loss = 0;
                }
            });
            if (valDataLoader) {
                let test_loss = 0;
                let accuracy = 0;
                model.model.eval();
                enumerate(valDataLoader, 0).forEach((data) => {
                    let inputs = data['image'];
                    let labels = data['label'];
                    const logps = model.model(inputs);
                    const batch_loss = model.criterion(logps, labels);
                    test_loss += batch_loss.item();
                    const ps = torch.exp(logps);
                    const [top_p, top_class] = ps.topk(1, boa.kwargs({ dim: 1 }));
                    const labelTensor = labels.view(...top_class.shape);
                    const equals = torch.eq(labelTensor, top_class);
                    accuracy +=
                        torch.mean(equals.type(torch.FloatTensor)).item();
                });
                console.log(`[epoch ${i}] validation loss: ${test_loss / len(valDataLoader)}, accuracy: ${accuracy / len(valDataLoader)}`);
            }
        }
    }
    yield saveModel((modelPath) => __awaiter(void 0, void 0, void 0, function* () {
        yield torch.save(model.model.state_dict(), path.join(modelPath, 'final.pth'));
    }));
    return model;
});
exports.default = modelTrain;
//# sourceMappingURL=index.js.map